import pandas as pd
import os
import xlrd
import openpyxl
from openpyxl.utils.cell import column_index_from_string
from openpyxl.utils import get_column_letter
from xlutils.copy import copy
import xlwings as xw
import time

start = time.time()
# FILE = "01vlookup.xls"  # .xls 파일 경로
# FILE = "01vlookup.xlsx"  # .xlsx 파일 경로
FILE = "01vlookup.xlsm"  # .xlsm 파일 경로

# df1 설정
SOURCE_SH = "Sheet1"
COL_START = "A"
COL_END = "B"

file_ext = os.path.splitext(FILE)[1]

COUNT_SKIPROW = 0  # 초기화

if file_ext == '.xls':
    wb = xlrd.open_workbook(FILE)
    ws = wb.sheet_by_name(SOURCE_SH)

    for row in range(ws.nrows):
        if ws.cell_value(row, 0) == "이름":
            break
        COUNT_SKIPROW += 1

    df1 = pd.read_excel(FILE, sheet_name=SOURCE_SH,
                        header=COUNT_SKIPROW,
                        usecols=f"{COL_START}:{COL_END}",
                        engine='xlrd')

elif file_ext == '.xlsx':
    FIND_COL = "A"
    FIND_TITLE = "이름"
    COUNT_SKIPROW = 0

    wb = openpyxl.load_workbook(FILE)
    ws = wb[SOURCE_SH]
    for row in ws.iter_rows(min_col=column_index_from_string(FIND_COL), max_col=column_index_from_string(FIND_COL)):
        if row[0].value == FIND_TITLE:
            break
        COUNT_SKIPROW += 1

    df1 = pd.read_excel(FILE, sheet_name=SOURCE_SH,
                        header=COUNT_SKIPROW,
                        usecols=f"{COL_START}:{COL_END}",
                        engine='openpyxl')

elif file_ext == '.xlsm':
    app = xw.App(visible=False)
    wb = app.books.open(FILE)
    ws = wb.sheets[SOURCE_SH]

    header_row = 1
    while ws.range(f'A{header_row}').value != "이름":
        header_row += 1

    last_row = ws.range('A4').end('down').row
    df1 = pd.DataFrame(ws.range(f"{COL_START}{header_row}:{COL_END}{last_row}").value)
    df1.columns = df1.iloc[0]
    df1 = df1[1:]

    wb.close()
    app.quit()

else:
    raise ValueError("지원하지 않는 파일 형식입니다.")

FIND_COL = "F"
FIND_TITLE = "제품"
COUNT_SKIPROW = 0

if file_ext == '.xls':
    for row in range(ws.nrows):
        if ws.cell_value(row, 5) == FIND_TITLE:
            break
        COUNT_SKIPROW += 1

elif file_ext == '.xlsx':
    for row in ws.iter_rows(min_col=column_index_from_string(FIND_COL), max_col=column_index_from_string(FIND_COL)):
        if row[0].value == FIND_TITLE:
            break
        COUNT_SKIPROW += 1

elif file_ext == '.xlsm':
    app = xw.App(visible=False)
    wb = app.books.open(FILE)
    ws = wb.sheets[SOURCE_SH]
    while ws.range(f'F{COUNT_SKIPROW + 1}').value != FIND_TITLE:
        COUNT_SKIPROW += 1
    wb.close()
    app.quit()

else:
    raise ValueError("지원하지 않는 파일 형식입니다.")

COL_START = "F"
COL_END = "G"
COL_TITLE_NAME = ['제품', '가격']

if file_ext == '.xls':
    df2 = pd.read_excel(FILE, sheet_name=SOURCE_SH,
                        header=0,
                        names=COL_TITLE_NAME,
                        index_col=None,
                        usecols=f"{COL_START}:{COL_END}",
                        skiprows=COUNT_SKIPROW,
                        engine='xlrd')

elif file_ext == '.xlsx' or file_ext == '.xlsm':
    df2 = pd.read_excel(FILE, sheet_name=SOURCE_SH,
                        header=0,
                        names=COL_TITLE_NAME,
                        index_col=None,
                        usecols=f"{COL_START}:{COL_END}",
                        skiprows=COUNT_SKIPROW,
                        engine='openpyxl')

# df3 데이터 병합
COL_STANDARD_NAME = "제품"
JOIN_HOW = "left"
df3 = df1.merge(df2, on=COL_STANDARD_NAME, how=JOIN_HOW)
df3['가격'] = df3['가격'].astype(int)

# 특정 범위 지우기
COL_START = "J"
COL_END = "L"
START_ROW = 4

if file_ext == '.xls':
    wb = xlrd.open_workbook(FILE, formatting_info=True)
    ws = wb.sheet_by_name(SOURCE_SH)
    new_wb = copy(wb)
    new_ws = new_wb.get_sheet(0)

    for column_letter in range(column_index_from_string(COL_START), column_index_from_string(COL_END) + 1):
        for row in range(START_ROW, ws.nrows):
            new_ws.write(row, column_letter - 1, '')

    new_wb.save(FILE)

elif file_ext == '.xlsx' or file_ext == '.xlsm':
    app = xw.App(visible=False)
    wb = app.books.open(FILE)
    ws = wb.sheets[SOURCE_SH]
    START_CELL = f'A{START_ROW}'
    last_row = ws.range(START_CELL).end('down').row

    if last_row == START_ROW:
        print("삭제할 데이터가 없습니다.")
    else:
        for column_letter in range(column_index_from_string(COL_START), column_index_from_string(COL_END) + 1):
            column_cells = ws.range(f'{get_column_letter(column_letter)}{START_ROW + 1}:'
                                    f'{get_column_letter(column_letter)}{last_row}')
            column_cells.value = None

    wb.save()
    wb.close()
    app.quit()

# 최종 파일 저장
if file_ext == '.xls':
    wb = xlrd.open_workbook(FILE, formatting_info=True)
    new_wb = copy(wb)
    new_ws = new_wb.get_sheet(0)

    for row_idx, row in df3.iterrows():
        for col_idx, value in enumerate(row):
            new_ws.write(START_ROW + row_idx, column_index_from_string(COL_START) - 1 + col_idx, value)

    new_wb.save(FILE)

    end = time.time()
    print("xls 경과시간:"f"{end - start:.5f} sec")

elif file_ext == '.xlsx':
    with pd.ExcelWriter(FILE, mode='a', engine='openpyxl', if_sheet_exists='overlay') as w:
        df3.to_excel(w, sheet_name=SOURCE_SH, index=False, header=False, startcol=column_index_from_string(COL_START) - column_index_from_string("A"), startrow=START_ROW)
    
    end = time.time()
    print("xlsx 경과시간:"f"{end - start:.5f} sec")

elif file_ext == '.xlsm':
    app = xw.App(visible=False)
    wb = app.books.open(FILE)
    ws = wb.sheets[SOURCE_SH]

    for row_idx, row in df3.iterrows():
        ws.range(f"{COL_START}{START_ROW + 1 + row_idx}").value = row.values.tolist()

    wb.save()
    wb.close()
    app.quit()

    end = time.time()
    print("xlsm 경과시간:"f"{end - start:.5f} sec")
